@@ -1,10 +1,10 @@
 /*
+    MythTV FUSE Module
+    Copyright (C) 2005-2008  Kees Cook <kees@outflux.net>
+
     FUSE: Filesystem in Userspace
     Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
 
-    MythTV FUSE Module
-    Copyright (C) 2005-2007  Kees Cook <kees@outflux.net>
-
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
 
@@ -81,6 +81,8 @@ struct mythtvfs {
     char * host;
     int    port;
     int short_display_flag;
+    int backend_version_override; // manually override compile-time defaults
+    int program_width_override;   // manually override compile-time defaults
     char * logfile_name;
 
     /* runtime state trackers */
@@ -96,10 +98,12 @@ static struct mythtvfs mythtvfs;
 
 static struct fuse_opt mythtvfs_opts[] = {
     /* -o options */
-    MYTHTVFS_OPT("host=%s",         host, 0),
-    MYTHTVFS_OPT("port=%d",         port, 0),
-    MYTHTVFS_OPT("short-display",   short_display_flag, 1),
-    MYTHTVFS_OPT("logfile=%s",      logfile_name, 0),
+    MYTHTVFS_OPT("host=%s",            host, 0),
+    MYTHTVFS_OPT("port=%d",            port, 0),
+    MYTHTVFS_OPT("short-display",      short_display_flag, 1),
+    MYTHTVFS_OPT("logfile=%s",         logfile_name, 0),
+    MYTHTVFS_OPT("backend-version=%d", backend_version_override, 0),
+    MYTHTVFS_OPT("program-width=%d",   program_width_override, 0),
   
     FUSE_OPT_KEY("-V",                KEY_VERSION),
     FUSE_OPT_KEY("--version",         KEY_VERSION),
@@ -111,7 +115,7 @@ static struct fuse_opt mythtvfs_opts[] = {
 static void usage(const char *progname)
 {
     fprintf(stderr,
-        "Usage: %s [options] original-path mythtvfs-mount\n"
+        "Usage: %s [options] -o host=BACKEND original-path mythtvfs-mount\n"
         "\n"
         "general options:\n"
         "    -o opt,[opt...]        mount options\n"
@@ -123,6 +127,8 @@ static void usage(const char *progname)
         "    -o port=PORT          mythtv backend host port\n"
         "    -o short-display      shorter filename display\n"
         "    -o logfile=FILE       write verbose logging to FILE\n"
+        "    -o backend-version=N  override compile-time backend version\n"
+        "    -o program-width=N    override compile-time program width\n"
         "\n",progname);
 }
 
@@ -340,46 +346,82 @@ void backend_client_check()
 #define MYTH_SEP "[]:[]"
 #define MYTH_PROGRAM_COLS  (mythtvfs.program_cols)
 
-void backend_version_check()
+int _version_check(int requested_version)
 {
     char *check = NULL, *result, *version="unknown";
-    char * versions[] = { "31", "26", "15", NULL };
     char version_request[80];
-    int idx;
-    int found=0;
-    
-    for (idx = 0; !found && versions[idx]; idx++) {
-        snprintf(version_request, 80, "MYTH_PROTO_VERSION %s", versions[idx]);
-        if (!backend_cmd(version_request, &check)) {
+    int rc = 0;
+
+    snprintf(version_request, 80, "MYTH_PROTO_VERSION %d", requested_version);
+    if (!backend_cmd(version_request, &check)) {
             fprintf(stderr,"Cannot check backend version!\n");
             exit(1);
-        }
-        result = check;
-        version = strstr(check,MYTH_SEP);
-        *version = '\0';
-        version += strlen(MYTH_SEP);
-        if (strcmp(result,"ACCEPT")) {
-            backend_close();
-            continue;
-        }
-
-        found = 1;
-        switch (atoi(version)) {
-            case 15: mythtvfs.program_cols = 39; break;
-            case 26: mythtvfs.program_cols = 41; break;
-            case 31: mythtvfs.program_cols = 42; break;
-            default:
-                fprintf(stderr,"Unknown protocol version (backend=%s)\n",version);
-                found = 0;
-        }
     }
-
-    if (!found) {
-        fprintf(stderr,"Could not find compatible protocol version (mythtvfs=%s, backend=%s)\n",versions[0],version);
-        exit(1);
+    result = check;
+    version = strstr(check,MYTH_SEP);
+    *version = '\0';
+    version += strlen(MYTH_SEP);
+    rc = atoi(version);
+    if (strcmp(result,"ACCEPT")) {
+        backend_close();
+        rc = 0 - rc;
     }
 
     if (check) free(check);
+    return rc;
+}
+
+struct backend_widths {
+    int version;
+    int width;
+};
+
+void backend_version_check()
+{
+    struct backend_widths versions[] = {
+        { 40, 46 },
+        { 31, 42 },
+        { 26, 41 },
+        { 15, 39 },
+        { 0,  0  },
+    };
+    int idx, version = 0;
+    int found=0;
+
+    // check for override version first
+    if (mythtvfs.backend_version_override &&
+        (version = _version_check(mythtvfs.backend_version_override))>0) {
+        mythtvfs.program_cols = versions[0].width;
+        found = 1;
+    }
+    
+    // check for known versions
+    for (idx = 0; !found && versions[idx].version; idx++) {
+        if ( (version = _version_check(versions[idx].version))<0 ) continue;
+
+        mythtvfs.program_cols = versions[idx].width;
+        found = 1;
+    }
+
+    if (!found) {
+        version = -version;
+        if (mythtvfs.program_width_override) {
+            fprintf(stderr,"Warning: unfamiliar backend version (%d), attempting width %d anyway.\n",version, mythtvfs.program_width_override);
+            if (_version_check(version) == version) {
+                mythtvfs.program_cols = mythtvfs.program_width_override;
+                found = 1;
+            }
+        }
+
+        if (!found) {
+            fprintf(stderr,"Could not find compatible protocol version (mythtvfs=%d, backend=%d)\nPerhaps try with '-o program-width=N' where N is the new protocol width?\n",versions[0].version,version);
+            exit(1);
+        }
+    }
+
+    if (mythtvfs.program_width_override) {
+        mythtvfs.program_cols = mythtvfs.program_width_override;
+    }
 }
 
 void backend_done()
@@ -539,7 +581,7 @@ static int mythtvfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     char * buffer = backend_query();
     char * data = buffer;
     while ( (data = backend_pop_program(data,&show)) ) {
-        char * metaname = NULL;
+        char *metaname = NULL, *ptr = NULL;
 
         /*
         printf("Title   : %s\n",show.title);
@@ -636,6 +678,9 @@ static int mythtvfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
                      filename);
         }
 
+        // filter "/" out of metaname
+        for (ptr = metaname; *ptr; ptr++) { if (*ptr == '/') *ptr='-'; }
+
         st.st_ino++;
         st.st_mode = S_IFREG|S_IRUSR|S_IRGRP|S_IROTH;
 
@@ -1064,6 +1109,8 @@ int main(int argc, char *argv[])
     mythtvfs.host      = NULL;
     mythtvfs.port      = 6543;
     mythtvfs.short_display_flag = 0;
+    mythtvfs.backend_version_override = 0;
+    mythtvfs.program_width_override = 0;
     mythtvfs.backend   = -1;
     mythtvfs.base_len  = 0;
     mythtvfs.logfile   = NULL;
